@@ -41,6 +41,8 @@
 #include "dc_reg.h"
 #include "dc_priv.h"
 
+static void _tegra_dc_disable(struct tegra_dc *dc);
+
 static int no_vsync;
 
 module_param_named(no_vsync, no_vsync, int, S_IRUGO | S_IWUSR);
@@ -778,57 +780,151 @@ int tegra_dc_sync_windows(struct tegra_dc_win *windows[], int n)
 }
 EXPORT_SYMBOL(tegra_dc_sync_windows);
 
-static unsigned long tegra_dc_pclk_round_rate(struct tegra_dc *dc, int pclk)
+static bool tegra_dc_valid_pixclock(const struct tegra_dc *dc,
+				    const struct fb_videomode *mode)
 {
-	unsigned long rate;
-	unsigned long div;
+	return PICOS2KHZ(mode->pixclock) <= dc->out->max_pclk_khz;
+}
 
-	rate = clk_get_rate(dc->clk);
+/*
+ * Find the best divider and resulting clock given an input clock rate and
+ * desired pixel clock, taking into account restrictions on the divider and
+ * output device.
+ */
+static unsigned long tegra_dc_pclk_best_div(const struct tegra_dc *dc,
+					    int pclk,
+					    unsigned long input_rate)
+{
+	/* Multiply by 2 since the divider works in .5 increments */
+	unsigned long div = DIV_ROUND_CLOSEST(input_rate * 2, pclk);
 
-	div = DIV_ROUND_CLOSEST(rate * 2, pclk);
-
-	if (div < 2)
+	if (!div)
 		return 0;
 
-	return rate * 2 / div;
+	/* Don't attempt to exceed this output's maximum pixel clock */
+	WARN_ON(!dc->out->max_pclk_khz);
+	while (input_rate * 2 / div > dc->out->max_pclk_khz * 1000)
+		div++;
+
+	/* We have a u7.1 divider, where 0 means "divide by 1" */
+	if (div < 2)
+		div = 2;
+	if (div > 257)
+		div = 257;
+
+	return div;
+}
+
+static unsigned long tegra_dc_pclk_round_rate(const struct tegra_dc *dc,
+					      int pclk,
+					      unsigned long *div_out)
+{
+	long rate = clk_round_rate(dc->clk, pclk);
+	unsigned long div;
+
+	if (rate < 0)
+		rate = clk_get_rate(dc->clk);
+
+	div = tegra_dc_pclk_best_div(dc, pclk, rate);
+
+	*div_out = div;
+
+	return rate;
+}
+
+static unsigned long tegra_dc_find_pll_d_rate(const struct tegra_dc *dc,
+					      unsigned long pclk,
+					      unsigned long *rate_out,
+					      unsigned long *div_out)
+{
+	/*
+	 * These are the only freqs we can get from pll_d currently.
+	 * TODO: algorithmically determine pll_d's m, n, p values so it can
+	 * output more frequencies.
+	 */
+	const unsigned long pll_d_freqs[] = {
+		216000000,
+		252000000,
+		594000000,
+		1000000000,
+	};
+	long best_pclk_ratio = 0;
+	unsigned long best_pclk = 0;
+	unsigned long best_rate = 0;
+	unsigned long best_div = 0;
+	int i;
+
+	if (dc->out->type != TEGRA_DC_OUT_HDMI)
+		return pclk;
+
+	for (i = 0; i < ARRAY_SIZE(pll_d_freqs); i++) {
+		const unsigned long rate = pll_d_freqs[i];
+		unsigned long rounded, div;
+		long ratio;
+		u64 tmp;
+
+		/* Divide rate by 2 since pll_d_out0 is always 1/2 pll_d */
+		div = tegra_dc_pclk_best_div(dc, pclk, rate / 2);
+		if (!div)
+			continue;
+
+		rounded = rate / div;
+		if (rounded > dc->out->max_pclk_khz * 1000)
+			continue;
+
+		tmp = (u64)rounded * 1000;
+		do_div(tmp, pclk);
+		ratio = lower_32_bits(tmp);
+
+		/* Ignore anything outside of 95%-105% of the target */
+		if (ratio < 950 || ratio > 1050)
+			continue;
+
+		if (abs(ratio - 1000) < abs(best_pclk_ratio - 1000)) {
+			best_pclk = rounded;
+			best_pclk_ratio = ratio;
+			best_rate = rate;
+			best_div = div;
+		}
+	}
+	if (rate_out)
+		*rate_out = best_rate;
+	if (div_out)
+		*div_out = best_div;
+
+	return best_pclk;
 }
 
 void tegra_dc_setup_clk(struct tegra_dc *dc, struct clk *clk)
 {
-	int pclk;
+	/*
+	 * We should always have a valid rate here, since modes should
+	 * go through tegra_dc_set_mode() before attempting to program them.
+	 */
+	WARN_ON(!dc->pll_rate);
 
 	if (dc->out->type == TEGRA_DC_OUT_HDMI) {
-		unsigned long rate;
 		struct clk *pll_d_out0_clk =
 			clk_get_sys(NULL, "pll_d_out0");
 		struct clk *pll_d_clk =
 			clk_get_sys(NULL, "pll_d");
 
-		if (dc->mode.pclk > 70000000)
-			rate = 594000000;
-		else if (dc->mode.pclk >= 27000000)
-			rate = 216000000;
-		else
-			rate = 252000000;
-
-		if (rate != clk_get_rate(pll_d_clk))
-			clk_set_rate(pll_d_clk, rate);
+		if (dc->pll_rate != clk_get_rate(pll_d_clk))
+			clk_set_rate(pll_d_clk, dc->pll_rate);
 
 		if (clk_get_parent(clk) != pll_d_out0_clk)
 			clk_set_parent(clk, pll_d_out0_clk);
+	} else {
+		tegra_dvfs_set_rate(clk, dc->pll_rate);
 	}
 
-	pclk = tegra_dc_pclk_round_rate(dc, dc->mode.pclk);
-	tegra_dvfs_set_rate(clk, pclk);
-
 }
 
-static int tegra_dc_program_mode(struct tegra_dc *dc, struct tegra_dc_mode *mode)
+static int tegra_dc_program_mode(struct tegra_dc *dc)
 {
+	const struct tegra_dc_mode *mode = &dc->mode;
 	unsigned long val;
-	unsigned long rate;
 	unsigned long div;
-	unsigned long pclk;
 
 	tegra_dc_writel(dc, 0x0, DC_DISP_DISP_TIMING_OPTIONS);
 	tegra_dc_writel(dc, mode->h_ref_to_sync | (mode->v_ref_to_sync << 16),
@@ -873,20 +969,8 @@ static int tegra_dc_program_mode(struct tegra_dc *dc, struct tegra_dc_mode *mode
 
 	tegra_dc_writel(dc, val, DC_DISP_DISP_INTERFACE_CONTROL);
 
-	rate = clk_get_rate(dc->clk);
-
-	pclk = tegra_dc_pclk_round_rate(dc, mode->pclk);
-	if (pclk < (mode->pclk / 100 * 99) ||
-	    pclk > (mode->pclk / 100 * 109)) {
-		dev_err(&dc->ndev->dev,
-			"can't divide %ld clock to %d -1/+9%% %ld %d %d\n",
-			rate, mode->pclk,
-			pclk, (mode->pclk / 100 * 99),
-			(mode->pclk / 100 * 109));
-		return -EINVAL;
-	}
-
-	div = (rate * 2 / pclk) - 2;
+	WARN_ON(dc->divider < 2 || dc->divider > 257);
+	div = dc->divider - 2;
 
 	tegra_dc_writel(dc, 0x00010001,
 			DC_DISP_SHIFT_CLOCK_OPTIONS);
@@ -896,11 +980,100 @@ static int tegra_dc_program_mode(struct tegra_dc *dc, struct tegra_dc_mode *mode
 	return 0;
 }
 
+bool tegra_dc_round_pclk(const struct tegra_dc *dc, struct fb_videomode *mode)
+{
+	unsigned long pclk_hz;
+
+	pclk_hz = PICOS2KHZ(mode->pixclock) * 1000;
+
+	if (dc->out->type == TEGRA_DC_OUT_HDMI) {
+		pclk_hz = tegra_dc_find_pll_d_rate(dc, pclk_hz, NULL, NULL);
+	} else {
+		unsigned long pll_rate, div;
+		pll_rate = tegra_dc_pclk_round_rate(dc, pclk_hz, &div);
+		pclk_hz = div ? (pll_rate * 2 / div) : 0;
+	}
+
+	if (!pclk_hz)
+		return false;
+
+	mode->pixclock = KHZ2PICOS(pclk_hz / 1000);
+
+	return true;
+}
+
+bool tegra_dc_mode_filter(const struct tegra_dc *dc,
+			  struct fb_videomode *mode)
+{
+	if (mode->vmode & FB_VMODE_INTERLACED)
+		return false;
+
+	/* ignore modes with a 0 pixel clock */
+	if (!mode->pixclock)
+		return false;
+
+	if (!tegra_dc_round_pclk(dc, mode)) {
+		dev_vdbg(&dc->ndev->dev, "MODE:%ux%u pclk(%lu) can't round\n",
+			 mode->xres, mode->yres,
+			 PICOS2KHZ(mode->pixclock) * 1000);
+		return false;
+	}
+
+	if (!tegra_dc_valid_pixclock(dc, mode)) {
+		dev_vdbg(&dc->ndev->dev, "MODE:%ux%u pclk(%lu) out of range\n",
+			 mode->xres, mode->yres,
+			 PICOS2KHZ(mode->pixclock) * 1000);
+		return false;
+	}
+
+	/* check some of DC's constraints */
+	if (mode->hsync_len > 1 && mode->vsync_len > 1 &&
+		mode->lower_margin + mode->vsync_len + mode->upper_margin > 1 &&
+		mode->xres >= 16 && mode->yres >= 16) {
+
+		dev_vdbg(&dc->ndev->dev, "MODE:%ux%u pclk(%lu)\n",
+			mode->xres, mode->yres,
+			PICOS2KHZ(mode->pixclock) * 1000);
+		return true;
+
+	}
+
+	dev_vdbg(&dc->ndev->dev, "rejecting MODE:%ux%u pclk(%lu)\n",
+		mode->xres, mode->yres, PICOS2KHZ(mode->pixclock) * 1000);
+
+	return false;
+}
+EXPORT_SYMBOL(tegra_dc_mode_filter);
 
 int tegra_dc_set_mode(struct tegra_dc *dc, const struct tegra_dc_mode *mode)
 {
+	unsigned long new_pclk = mode->pclk;
+	unsigned long pll_rate, div;
+
+	if (!new_pclk) {
+		memset(&dc->mode, 0, sizeof(dc->mode));
+		return 0;
+	}
+
+	if (dc->out->type == TEGRA_DC_OUT_HDMI) {
+		new_pclk = tegra_dc_find_pll_d_rate(dc, new_pclk,
+						    &pll_rate, &div);
+
+	} else {
+		pll_rate = tegra_dc_pclk_round_rate(dc, new_pclk, &div);
+		new_pclk = div ? (pll_rate * 2 / div) : 0;
+	}
+
+	if (!new_pclk)
+		return -EINVAL;
+
+	dc->pll_rate = pll_rate;
+	dc->divider = div;
+
 	memcpy(&dc->mode, mode, sizeof(dc->mode));
 
+	dc->mode.pclk = new_pclk;
+
 	return 0;
 }
 EXPORT_SYMBOL(tegra_dc_set_mode);
@@ -909,6 +1082,9 @@ static void tegra_dc_set_out(struct tegra_dc *dc, struct tegra_dc_out *out)
 {
 	dc->out = out;
 
+	if (!dc->out->max_pclk_khz)
+		dc->out->max_pclk_khz = ULONG_MAX;
+
 	if (out->n_modes > 0)
 		tegra_dc_set_mode(dc, &dc->out->modes[0]);
 
@@ -928,7 +1104,6 @@ static void tegra_dc_set_out(struct tegra_dc *dc, struct tegra_dc_out *out)
 
 	if (dc->out_ops && dc->out_ops->init)
 		dc->out_ops->init(dc);
-
 }
 
 unsigned tegra_dc_get_out_height(struct tegra_dc *dc)
@@ -949,6 +1124,12 @@ unsigned tegra_dc_get_out_width(struct tegra_dc *dc)
 }
 EXPORT_SYMBOL(tegra_dc_get_out_width);
 
+const struct tegra_dc_mode *tegra_dc_get_current_mode(const struct tegra_dc *dc)
+{
+	return &dc->mode;
+}
+EXPORT_SYMBOL(tegra_dc_get_current_mode);
+
 static irqreturn_t tegra_dc_irq(int irq, void *ptr)
 {
 	struct tegra_dc *dc = ptr;
@@ -1140,7 +1321,7 @@ static u32 get_syncpt(struct tegra_dc *dc, int idx)
 	return syncpt_id;
 }
 
-static void tegra_dc_init(struct tegra_dc *dc)
+static int tegra_dc_init(struct tegra_dc *dc)
 {
 	int i;
 
@@ -1206,11 +1387,16 @@ static void tegra_dc_init(struct tegra_dc *dc)
 	}
 
 	if (dc->mode.pclk)
-		tegra_dc_program_mode(dc, &dc->mode);
+		if (tegra_dc_program_mode(dc))
+			return -EINVAL;
+
+	return 0;
 }
 
 static bool _tegra_dc_enable(struct tegra_dc *dc)
 {
+	int failed_init;
+
 	if (dc->mode.pclk == 0)
 		return false;
 
@@ -1228,7 +1414,7 @@ static bool _tegra_dc_enable(struct tegra_dc *dc)
 
 	enable_irq(dc->irq);
 
-	tegra_dc_init(dc);
+	failed_init = tegra_dc_init(dc);
 
 	if (dc->out_ops && dc->out_ops->enable)
 		dc->out_ops->enable(dc);
@@ -1236,6 +1422,11 @@ static bool _tegra_dc_enable(struct tegra_dc *dc)
 	/* force a full blending update */
 	dc->blend.z[0] = -1;
 
+	if (failed_init) {
+		_tegra_dc_disable(dc);
+		return false;
+	}
+
 	tegra_dc_ext_enable(dc->ext);
 
 	return true;
